; Growing plant with music in black and white
; The music tempo is unfortunately not always constant because it's synced with drawing and not based on interrupt/VBLANK

USE_UNDOCUMENTED equ 1
SAFE_VERSION equ 1

COLOR0	equ $02c4
COLOR1	equ $02c5
COLOR2	equ $02c6
COLOR3	equ $02c7
COLOR4	equ $02c8

RANDOM	equ $d20a

;;;;;; SOUND 
AUDF1	EQU	$D200	;channel 1 audio frequency
AUDC1	EQU	$D201	;channel 1 audio control

AUDF2	EQU	$D202	;channel 2 audio frequency
AUDC2	EQU	$D203	;channel 2 audio control

AUDF3	EQU	$D204	;channel 3 audio frequency
AUDC3	EQU	$D205	;channel 3 audio control

AUDF4	EQU	$D206	;channel 4 audio frequency
AUDC4	EQU	$D207	;channel 4 audio control

AUDCTL	EQU	$D208	;audio control
SKCTL   equ $D20F


; OS functions
openmode  equ $ef9c
drawpoint equ $f1d8
ROWCRS	equ $54		;Row of cursor, 1 byte
COLCRS	equ $55		;Column of cursor, 2 bytes
COLOR   equ $2fb	;Color for graphics operations

; Aliases, because orig reg names are hard to remember ;)
cursor_y    equ ROWCRS
cursor_x    equ COLCRS

POINTS equ 14 ; /2

	org $80
pos_lo .ds[POINTS] ; *2 because of y and x positions
pos_hi .ds[POINTS] ; *2

;;;;;;;;;; THE REST HAVE POINTS*2 to simplify calculations

; mask that triggers change direction
angle_change_mask .ds[POINTS]

; angle to be added -1,+1
angle_delta .ds[POINTS]

; at which angle the point is moving
angle .ds[POINTS]

; at the beginning all points follow the main one, but have different number of steps
; when steps=0 then position is getting reset to the main one
; main one steps do not decrease
steps .ds[POINTS]

.print "*** END OF POINTS: ", *

global_step .ds[1]
temp .ds [1]

;point_index .byte 0
point_index equ $4F ; gets zeroed by setting GFX mode, will work until ATRACT is on

;music_step .byte 0
music_step equ $7B ; gets zeroed by setting GFX mode


.IF SAFE_VERSION = 0
	bvc start
.ENDIF

MUSIC_data: 

;Track $01
MUSIC_track01
	dta $79,$66,$51,$79,$66,$51,$79,$51
;Track $02
MUSIC_track02
	dta $88,$6C,$5B,$88,$6C,$5B,$88,$5B
;Track $03
MUSIC_track03
	dta $90,$79,$5B,$90,$79,$5B,$90,$5B
;Track $04
MUSIC_track04
	dta $80,$6C,$51,$80,$6C,$51,$6C,$51

.print "*** END OF MUSIC: ", *

;	org $3000
; http://forums.nesdev.com/viewtopic.php?t=20028
.proc add16signed
	; in ACC signed value to add
	bpl notneg
	dec pos_hi,x
notneg:
;	clc
	adc pos_lo,x
	sta pos_lo,x
	bcc exit
	inc pos_hi,x
exit
	rts
.endp

start:
	lda #8
	jsr openmode

	sty COLOR1

	lda #$0F
	sta COLOR2
	
.local init_sine
;	 clc
;	 ldy #$0
;	 lda #$15
	 ldx #0
LOOP1
	 dey
LOOP2
	 pha
	 eor #$80 ; position to the center
	 sta sine,x
	 sta sine+64,x ; extend table for cosine
	 pla
	 
	 dex 
	 beq exit

	 sty temp
	 adc temp
	 bmi LOOP1
	 iny
	 jmp LOOP2
EXIT:
.endl
	; A = $25
	; X = $00
	; Y = $07
	
main_loop:	
	ldx point_index
	
	dec steps,x
	bne more_steps

	;;;; no more steps for the point
	; set position to main branch
	lda pos_hi+1
	sta pos_hi+1,x
	lda pos_hi
	sta pos_hi,x
	
	; set the same angle
	ldy angle
	sty angle,x

	lda RANDOM
	sta steps,x
	and #$1f
	sta angle_change_mask,x
	
	; set -1 or +1 to angle_delta
	and #%10
	clc
	sbc #0
	sta angle_delta,x
	
more_steps:	
	lda global_step
	and angle_change_mask,x
	bne skip_angle_change
	
	clc ; needed
	lda angle,x
	adc angle_delta,x
	sta angle,x
	
skip_angle_change:


.local move_by_angle
	ldy angle,x
	lda cosine,y ; y
	; a is "velocity" in this axis
	jsr add16signed
			
	inx
	lda sine,y ; x
	; a is "velocity" in this axis
	jsr add16signed
.endl


.local draw
;	dex
	lda pos_hi-1,x
	sta cursor_y
	lda pos_hi,x
	sta cursor_x
	inc color
	; check if the main branch
	cpx #1 ; because we removed dex at the beginning of draw
	bne exit
	; bigger branch
	jsr drawpoint
	inc cursor_x
	jsr drawpoint
	dec cursor_y

;	jsr drawpoint
;	dec cursor_x
exit
	jsr drawpoint
.endl
	; X,Y,A destroyed

.local shade_screen
	dec color
	lda #8 ; how many points we draw + controls music speed
	sta temp

shade_loop:
	lda RANDOM
	sta cursor_x
	lda RANDOM
	sta cursor_y
	jsr drawpoint
	dec temp
	bne shade_loop
.endl

	dec point_index
	dec point_index
main_loop_branch:
	bpl main_loop
		
.local play_note
	; play note
	; we play 4xeach of 4 tracks

; take note

	lda music_step

	pha	
	and #$7		
	tax ; x is note in pattern 0-7
	pla
	
.if USE_UNDOCUMENTED = 1
	alr #%01100000 ; and+lsr
.else
	and #%01100000
	lsr 
.endif
	lsr
	
	; add first pattern offset
	adc #<MUSIC_data ; TODO check if we can place music at $80 and just make ORA $80 in calculations
	sta music_part	
		
	lda MUSIC_data,x	
music_part equ *-1
	sta AUDF1
.endl

	;y=0
;UNDOCUMENTED OPCODES
.if USE_UNDOCUMENTED = 1
	tya
	isb global_step
.else
	dec global_step
	lda global_step
.endif
	and #$7
	bne skip_music_change
	inc music_step
skip_music_change

	adc #$A0
	sta AUDC1
		
	lda #POINTS-2
	sta point_index	

;	bpl main_loop ; branch out of range
;	jmp main_loop
	bpl main_loop_branch
	
	org $4000
sine .ds[128] ; 2*64
cosine equ sine+$10 ; +$10 with 64 sine period, +$40 with 256 sine period

.IF SAFE_VERSION = 1
	RUN start
.ENDIF